from mobicontrol.client.auth import authenticate
from mobicontrol.client import MobicontrolClient, ConfigurationError
import click


@click.group(invoke_without_command=True, name="mc")
@click.pass_context
def mobicontrol(ctx):
    try:
        ctx.obj = MobicontrolClient.load()
    except ConfigurationError as err:
        raise click.ClickException(str(err))

    if ctx.invoked_subcommand is None:
        ctx.obj.save()
        click.echo("Welcome to Mobicontrol CLI")
        click.echo(f"Your deployment server is located at {ctx.obj.base_url}")

    if ctx.obj.base_url is None:
        click.echo("Sorry, you need to configure the URL before using the CLI")


@mobicontrol.command()
@click.option("--url", envvar="MC_URL")
@click.option("--client_id", envvar="MC_CLIENT_ID")
@click.option("--client_secret", envvar="MC_CLIENT_SECRET")
@click.option("--username", envvar="MC_USERNAME")
@click.option("--password", envvar="MC_PASSWORD")
@click.pass_context
def login(ctx, url, client_id, client_secret, username, password):
    click.echo(f"Logging in as {username}")
    try:
        ctx.obj.base_url = url
        authenticate(ctx.obj, client_id, client_secret, username, password)
        ctx.obj.save()
    except Exception as e:
        click.echo(e)
        return

    click.echo("Successfully logged in!")


from . import apps, policies, apply



import requests
from mobicontrol.client import MobicontrolClient


def authenticate(
    client: MobicontrolClient,
    client_id: str,
    client_secret: str,
    username: str,
    password: str,
) -> None:
    auth = requests.models.HTTPBasicAuth(client_id, client_secret)

    response = client.post(
        "/token",
        auth=auth,
        data={"grant_type": "password", "username": username, "password": password},
    )

    if response.status_code != 200:
        raise Exception("Could not authenticate")

    client.headers["Authorization"] = f"Bearer {response.json()['access_token']}"



import base64


def get_file_contents(path: str) -> bytes:
    with open(path, "rb") as file:
        read_file = file.read()

        return base64.b64encode(read_file)


def get_filename_from_path(path: str) -> str:
    return path.split("/")[-1]



import requests
from typing import Union
import os
import json


class NotLoggedIn(Exception):
    pass


class ConfigurationError(Exception):
    pass


WORKDIR = os.path.expanduser("~/.mobicontrol")


class MobicontrolClient(requests.Session):
    def __init__(
        self, base_url: str, headers: Union[dict, None] = None, *args, **kwargs
    ) -> None:
        super().__init__(*args, **kwargs)
        self.base_url = base_url

        if headers is not None:
            self.headers = headers

    def request(
        self, method: Union[str, bytes], url: Union[str, bytes], *args, **kwargs
    ) -> requests.Response:

        if self.base_url not in url:
            url = f"{self.base_url}/MobiControl/api{str(url)}"

        return super().request(method, url, *args, **kwargs)

    def to_json(self) -> dict:
        return {"base_url": self.base_url, "headers": dict(self.headers)}

    def save(self) -> None:
        if not os.path.exists(WORKDIR):
            os.makedirs(WORKDIR)

        with open(f"{WORKDIR}/store.json", "w") as file:
            file.write(json.dumps(self.to_json()))

    @classmethod
    def load(cls) -> "MobicontrolClient":
        try:
            with open(f"{WORKDIR}/store.json", "r") as file:
                state = json.loads(file.read())

                return cls(**state)
        except Exception:
            raise ConfigurationError("Could not load configuration")



from mobicontrol.cli import mobicontrol
import click
import yaml
import json
from mobicontrol.client.policies import (
    get_policies,
    create_policy,
    set_apps,
    set_assignments,
    delete_policy,
)
from mobicontrol.client.apps import upload_app
import requests
import base64
import os


def get_github_release_asset(
    owner: str, repo: str, tag: str, filename: str = ".apk"
) -> tuple[str, bytes]:
    github_token = os.environ.get("GITHUB_TOKEN")

    if github_token is None:
        raise click.ClickException(
            "GITHUB_TOKEN environment variable is required when using github-release source type."
        )

    resp = requests.get(
        f"https://api.github.com/repos/{owner}/{repo}/releases",
        headers={
            "Accept": "application/vnd.github+json",
            "Authorization": f"Bearer {github_token}",
        },
    )

    if resp.status_code != 200:
        raise click.ClickException(
            f"Could not fetch releases from GitHub: {resp.text}",
        )

    releases: list = resp.json()

    for release in releases:
        if release["tag_name"] == tag:
            release = release
            break
    else:
        raise click.ClickException(f"Could not find release {tag}.")

    resp = requests.get(
        release["assets_url"],
        headers={
            "Accept": "application/vnd.github+json",
            "Authorization": f"Bearer {github_token}",
        },
    )

    if resp.status_code != 200:
        raise click.ClickException(
            f"Could not fetch assets from GitHub release: {resp.text}",
        )

    releases = resp.json()

    for asset in releases:
        if filename in asset["name"]:
            asset = asset
            break
    else:
        raise click.ClickException(f"Could not find asset matching '{filename}'.")

    resp = requests.get(
        asset["url"],
        headers={
            "Accept": "application/octet-stream",
            "Authorization": f"Bearer {github_token}",
        },
    )

    if resp.status_code != 200:
        raise click.ClickException(
            f"Could not fetch asset from GitHub release: {resp.text}",
        )

    return asset["name"], base64.b64encode(resp.content)


def apply_apps(ctx, policy_id: str, apps: list[dict]):
    for app in apps:
        if "source" in app:
            if app["source"]["type"] == "github-release":
                filename, file = get_github_release_asset(
                    app["source"]["owner"],
                    app["source"]["repo"],
                    app["source"]["tag"],
                    app["source"].get("filename", ".apk"),
                )

            mc_app = upload_app(ctx.obj, filename, file)

    payload = [
        {
            "ReferenceId": mc_app["ReferenceId"],
            "IsMandatory": app.get("mandatory", True),
            "AppPriority": app.get("priority", 1),
            "AppConfiguration": json.dumps(app.get("config", {})),
        }
        for app in apps
    ]

    set_apps(ctx.obj, policy_id, payload)
    click.echo(f"Assigned {len(payload)} apps to policy.")


def apply_policy(ctx, manifest: dict):
    meta: dict = manifest["metadata"]
    try:
        policies = get_policies(ctx.obj, meta["name"])

        for policy in policies:
            if policy["Name"] == meta["name"]:
                policy = policy
                click.echo(f"Found existing policy with name {meta['name']}.")
                break
        else:
            policy = create_policy(
                ctx.obj, meta["name"], meta["kind"], meta.get("description", "")
            )
            click.echo(f"Created new policy with name {meta['name']}.")
    except Exception as e:
        raise click.ClickException(str(e))

    apps = manifest.get("apps", [])

    try:
        apply_apps(ctx, policy["ReferenceId"], apps)

        assignment_groups = manifest.get("assignmentGroups", [])
        set_assignments(ctx.obj, policy["ReferenceId"], assignment_groups)
        click.echo(f"Assigned policy to {len(assignment_groups)} device groups.")
    except Exception as e:
        raise click.ClickException(str(e))


def delete_policy_manifest(ctx, manifest: dict):
    meta = manifest["metadata"]
    try:
        policies = get_policies(ctx.obj, meta["name"])

        for policy in policies:
            if policy["Name"] == meta["name"]:
                policy = policy
                break
        else:
            raise click.ClickException(
                f"Could not find policy with name {meta['name']}"
            )
    except Exception as e:
        raise click.ClickException(str(e))

    try:
        delete_policy(ctx.obj, policy["ReferenceId"])
    except Exception as e:
        raise click.ClickException(str(e))


def get_files(path: str) -> list[str]:
    if os.path.isdir(path):
        paths: list[str] = []
        for root, dirs, files in os.walk(path):
            for file in files:
                paths.append(os.path.join(root, file))
        return paths
    else:
        return [path]


@mobicontrol.command()
@click.option("--file", type=click.Path(exists=True), required=True)
@click.pass_context
def apply(ctx, file: str):
    apply_paths = get_files(file)

    click.echo(f"Found {len(apply_paths)} files to apply.")
    for file in apply_paths:
        click.echo(f"Applying {file}.")
        with open(file) as f:
            data = yaml.safe_load(f)

        if data["resourceType"] == "policy":
            apply_policy(ctx, data)

        click.echo(f"{file} applied.")


@mobicontrol.command()
@click.option("--file", type=click.Path(exists=True), required=True)
@click.pass_context
def delete(ctx, file: str):
    apply_paths = get_files(file)

    click.echo(f"Found {len(apply_paths)} files to delete.")
    for file in apply_paths:
        with open(file) as f:
            data = yaml.safe_load(f)

        if data["resourceType"] == "policy":
            try:
                delete_policy_manifest(ctx, data)
                click.echo(f"Deleted {file}.")
            except Exception as e:
                click.echo(str(e))



from mobicontrol.client import MobicontrolClient
from email.mime.multipart import MIMEMultipart
from email.mime.base import MIMEBase
import json

BOUNDARY = "mc_boundary"


def get_app(client: MobicontrolClient, app_id: str):
    response = client.get(f"/appManagement/android/apps//enterprise/{app_id}")

    if response.status_code != 200:
        raise Exception(f"Could not fetch app. Error code: {response.status_code}. {response.text}")

    return response.json()


def upload_app(client: MobicontrolClient, filename: str, file: bytes):
    message = MIMEMultipart("related", charset="utf-8")
    message.add_header("Content-Disposition", f"attachment; filename={filename}")

    # add metadata
    metadata = MIMEBase("application", "vnd.android.application.metadata+json")
    metadata.set_payload(json.dumps({"DeviceFamily": "AndroidPlus"}))
    del metadata["mime-version"]
    message.attach(metadata)

    # add file base 64 encoded
    binary_file = MIMEBase("application", "vnd.android.application")
    binary_file["Content-Type-Encoding"] = "base64"
    binary_file["Content-Disposition"] = f'attachment; filename="{filename}"'
    binary_file.set_payload(file)
    del binary_file["mime-version"]

    message.attach(binary_file)

    body = message.as_string().split("\n\n", 1)[1]
    body = body.replace("\n", "\r\n")

    headers = dict(message.items())

    response = client.post(
        "/appManagement/android/apps/enterprise/internal",
        headers={"Content-Type": headers["Content-Type"]},
        data=body.encode("utf-8"),
    )

    if response.status_code != 200:
        raise Exception(f"Upload failed with status code {response.status_code}. {response.text}.")

    data = response.json()

    if "ErrorCode" in data:
        raise Exception(f"Upload failed with message: {data['Message']}")

    return data



from mobicontrol.cli import mobicontrol
from mobicontrol.client.policies import (
    get_policies,
    create_policy,
    delete_policy,
    get_policy,
    add_app,
    get_apps,
    remove_app,
    get_assignments,
    assign_device_group,
    unassign_device_group,
)
import click
import json
from typing import Optional


@mobicontrol.group()
@click.option("--id", type=str, required=False)
@click.pass_context
def policy(ctx, id: Optional[str] = None):
    pass


@policy.command()
@click.pass_context
def list(ctx):
    try:
        policies = get_policies(ctx.obj)
    except Exception as e:
        raise click.ClickException(str(e))

    click.echo(json.dumps(policies))


@policy.command()
@click.option("--id", type=str, required=True)
@click.pass_context
def show(ctx, id: str):
    try:
        policy = get_policy(ctx.obj, id)
    except Exception as e:
        raise click.ClickException(f"Could not fetch policy. {e}")

    click.echo(json.dumps(policy))


@policy.command()
@click.option("--name", type=str, required=True)
@click.option("--kind", type=click.types.Choice(["ManagedGooglePlay"]), required=True)
@click.option("--description", type=str)
@click.pass_context
def create(ctx, name: str, kind: str, description: str = ""):
    try:
        policies = get_policies(ctx.obj, name_contains=name)
    except Exception as e:
        raise click.ClickException(f"Could not fetch policies. {e}")

    if len(policies) == 1:
        policy = policies[0]
    elif len(policies) > 1:
        raise click.ClickException(f"Found more than one policy with name {name}")
    else:
        policy = create_policy(ctx.obj, name, kind, description)

    click.echo(json.dumps(policy))


@policy.command()
@click.option("--id", type=str, required=True)
@click.pass_context
def delete(ctx, id: str):
    try:
        delete_policy(ctx.obj, id)
    except Exception as e:
        raise click.ClickException(f"Could not delete policy. {e}")


@policy.group()
def app():
    pass


@app.command("list")
@click.option("--id", type=str, required=True)
@click.pass_context
def list_apps(ctx, id: str):
    apps = get_apps(ctx.obj, id)

    click.echo(json.dumps(apps))


@app.command("create")
@click.option("--id", type=str, required=True)
@click.option("--app", type=str, required=True)
@click.option("--config", type=str, required=False)
@click.pass_context
def create_app(
    ctx, id: str, app: str, config: Optional[str] = None, remove: bool = False
):
    try:
        add_app(ctx.obj, id, app, config)
    except Exception as e:
        raise click.ClickException(f"Could not add app to policy. {e}")


@app.command("delete")
@click.option("--id", type=str, required=True)
@click.option("--app", type=str, required=True)
@click.pass_context
def delete_app(ctx, id: str, app: str):
    try:
        remove_app(ctx.obj, id, app)
    except Exception as e:
        raise click.ClickException(f"Could not remove app from policy. {e}")


@policy.group("assignment")
def assignment():
    pass


@assignment.command("list")
@click.option("--id", type=str, required=True)
@click.pass_context
def list_assignments(ctx, id: str):
    assignments = get_assignments(ctx.obj, id)

    if assignments is None:
        assignments = []

    click.echo(json.dumps(assignments))


@assignment.command("create")
@click.option("--id", type=str, required=True)
@click.option("--group", type=str, required=True)
@click.pass_context
def create_assignment(ctx, id: str, group: str):
    try:
        assign_device_group(ctx.obj, id, group)
    except Exception as e:
        raise click.ClickException(f"Could not assign policy to group. {e}")


@assignment.command("delete")
@click.option("--id", type=str, required=True)
@click.option("--group", type=str, required=True)
@click.pass_context
def delete_assignment(ctx, id: str, group: str):
    try:
        unassign_device_group(ctx.obj, id, group)
    except Exception as e:
        raise click.ClickException(f"Could not unassign policy from group. {e}")



from mobicontrol.cli import mobicontrol
import click
from mobicontrol.utils import get_file_contents, get_filename_from_path
from mobicontrol.client.apps import upload_app, get_app
import json


@mobicontrol.group()
def app():
    pass


@app.command()
@click.option("--id", type=str, required=True)
@click.pass_context
def show(ctx, id: str):
    try:
        app = get_app(ctx.obj, id)
    except Exception as e:
        raise click.ClickException(f"Could not fetch app. {e}")

    click.echo(json.dumps(app))


@app.command()
@click.option("--file", type=click.Path(exists=True), required=True)
@click.pass_context
def upload(ctx, file: str):
    filename = get_filename_from_path(file)
    content = get_file_contents(file)

    try:
        app = upload_app(ctx.obj, filename, content)
    except Exception as e:
        raise click.ClickException(f"Could not upload app. {e}")

    click.echo(json.dumps(app))



from setuptools import setup

setup(
    name="mobicontrol",
    version="1.3.3",
    packages=["mobicontrol", "mobicontrol.cli", "mobicontrol.client"],
    include_package_data=True,
    install_requires=["Click", "requests", "PyYAML"],
    entry_points={"console_scripts": ["mc = mobicontrol.cli:mobicontrol"]},
)



from mobicontrol.client import MobicontrolClient
from typing import Optional
import re


def get_policies(
    client: MobicontrolClient,
    name_contains: Optional[str] = None,
    families: Optional[str] = None,
    statuses: Optional[str] = None,
    is_scheduled: Optional[bool] = None,
    order: Optional[str] = None,
    skip: Optional[int] = None,
    take: Optional[int] = None,
) -> list:
    response = client.get(
        "/appManagement/policies",
        params={
            "nameContains": name_contains,
            "families": families,
            "statuses": statuses,
            "isScheduled": is_scheduled,
            "order": order,
            "skip": skip,
            "take": take,
            "source": "",
        },
    )

    if response.status_code != 200:
        raise Exception(
            f"Could not fetch policies. Error code: {response.status_code}. {response.text}"
        )

    return response.json()


def get_policy(
    client: MobicontrolClient,
    policy_id: str,
):
    response = client.get(f"/appManagement/android/policies/{policy_id}")

    if response.status_code != 200:
        raise Exception(
            f"Could not delete policy. Error code: {response.status_code}. {response.text}"
        )

    return response.json()


def create_policy(
    client: MobicontrolClient,
    name: str,
    kind: str,
    description: str = "",
) -> dict:
    response = client.post(
        "/appManagement/android/policies",
        json={
            "Name": name,
            "Description": description,
            "RuleKind": kind,
        },
    )

    if response.status_code != 200:
        raise Exception(
            f"Could not create policy. Error code: {response.status_code}. {response.text}"
        )

    return response.json()


def delete_policy(client: MobicontrolClient, policy_id: str):
    response = client.delete(f"/appManagement/android/policies/{policy_id}")

    if response.status_code != 204:
        raise Exception(
            f"Could not delete policy. Error code: {response.status_code}. {response.text}"
        )


def get_apps(client: MobicontrolClient, policy_id: str) -> list:
    response = client.get(
        f"/appManagement/android/policies/{policy_id}/apps/enterprise"
    )

    if response.status_code != 200:
        raise Exception(
            f"Could not fetch apps. Error code: {response.status_code}. {response.text}"
        )

    return response.json()


def set_apps(client: MobicontrolClient, policy_id: str, apps: list):
    response = client.put(
        f"/appManagement/android/policies/{policy_id}/apps/enterprise",
        json=apps,
    )

    if response.status_code != 204:
        data = response.json()

        if "ErrorCode" in data:
            raise Exception(f"Upload failed with message: {data['Message']}")

        raise Exception(
            f"Upload failed with status code {response.status_code}. {response.text}."
        )


def add_app(
    client: MobicontrolClient, policy_id: str, app_id: str, config: Optional[str] = ""
):
    apps = get_apps(client, policy_id)

    payload = []

    for app in apps:
        app_payload = {
            "ReferenceId": app["ReferenceId"],
            "IsMandatory": app["IsMandatory"],
            "AppPriority": app["AppPriority"],
            "AppConfiguration": app["AppConfiguration"],
        }

        if app["ReferenceId"] == app_id:
            app_payload["AppConfiguration"] = config

        payload.append(app_payload)

    if app_id not in [app["ReferenceId"] for app in apps]:
        payload.append(
            {
                "ReferenceId": app_id,
                "IsMandatory": True,
                "AppPriority": 1,
                "AppConfiguration": config,
            }
        )

    set_apps(client, policy_id, payload)


def remove_app(client: MobicontrolClient, policy_id: str, app_id: str):
    apps = get_apps(client, policy_id)

    payload = []
    for app in apps:
        if app["ReferenceId"] != app_id:
            payload.append(
                {
                    "ReferenceId": app["ReferenceId"],
                    "IsMandatory": app["IsMandatory"],
                    "AppPriority": app["AppPriority"],
                    "AppConfiguration": app["AppConfiguration"],
                }
            )

    set_apps(client, policy_id, payload)


def get_assignments(client: MobicontrolClient, policy_id: str) -> dict:
    response = client.get(f"/appManagement/policies/{policy_id}/assignment")

    if response.status_code != 200:
        raise Exception(f"Error code: {response.status_code}. {response.text}")

    data = response.json()

    if data is None:
        return {"TargetDeviceGroups": []}

    return data


def set_assignments(
    client: MobicontrolClient, policy_id: str, device_groups: list[str]
):

    groups_payload = [
        {
            "DeviceGroupPath": group,
            "Excluded": False,
        }
        for group in device_groups
    ]

    payload = {
        "TargetDeviceGroups": groups_payload,
    }

    response = client.put(
        f"/appManagement/policies/{policy_id}/assignment",
        json=payload,
    )

    if response.status_code != 200:
        raise Exception(f"Error code: {response.status_code}. {response.text}")

    return response.json()


def assign_device_group(client, policy_id: str, device_group: str):
    assignments = get_assignments(client, policy_id)

    group_ids = [
        assignment["DeviceGroupPath"]
        for assignment in assignments["TargetDeviceGroups"]
    ]

    if device_group in group_ids:
        return

    group_ids.append(device_group)

    set_assignments(client, policy_id, group_ids)


def unassign_device_group(client: MobicontrolClient, policy_id: str, device_group: str):
    assignments = get_assignments(client, policy_id)

    group_ids = [
        assignment["DeviceGroupPath"]
        for assignment in assignments["TargetDeviceGroups"]
    ]

    if device_group not in group_ids:
        return

    group_ids.remove(device_group)

    set_assignments(client, policy_id, group_ids)
